# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
#
#
#Modifications copyright (C) 2017 Malte Loller-Andersen
#
#


from __future__ import division
from __future__ import print_function

import collections
import os

import tensorflow as tf
import re

from nltk.tokenize import RegexpTokenizer

tokenizer = RegexpTokenizer(r'\w+')

#target = open('dataset/preprocessedA.txt', 'r').read().decode("utf-8").replace("\n", "<eos>").split()

#print(target)

#tokens = tokenizer.tokenize(target)

#print(tokens)

#pathtofile = "dataset/testproc.txt"

def _read_words(filename):
  with tf.gfile.GFile(filename, "r") as f:
    a = re.findall(r"[\w']+", f.read().decode("utf-8").replace("\n", "<eos>").lower())
    #print(a)
    return a


def _build_vocab(filename):
  data = _read_words(filename)

  counter = collections.Counter(data)
  count_pairs = sorted(counter.items(), key=lambda x: (-x[1], x[0]))

  words, _ = list(zip(*count_pairs))
  word_to_id = dict(zip(words, range(len(words))))

  return word_to_id


def _file_to_word_ids(filename, word_to_id):
  data = _read_words(filename)
  return [word_to_id[word] for word in data if word in word_to_id]


def lyr_raw_data(data_path="dataset/final_dataset/"):
  all_data = os.path.join(data_path, "preprocfull30000.txt")
  train_path = os.path.join(data_path, "preprocfull30000train.txt")
  valid_path = os.path.join(data_path, "preprocfull30000val.txt")
  test_path = os.path.join(data_path, "preprocfull30000test.txt")

  word_to_id = _build_vocab(all_data)
  train_data = _file_to_word_ids(train_path, word_to_id)
  valid_data = _file_to_word_ids(valid_path, word_to_id)
  test_data = _file_to_word_ids(test_path, word_to_id)
  vocabulary = len(word_to_id)
  #print(word_to_id)
  print(vocabulary)
  return train_data, valid_data, test_data, vocabulary, word_to_id

def lyr_producer(raw_data, batch_size, num_steps, name=None):
  with tf.name_scope(name, "LyrProducer", [raw_data, batch_size, num_steps]):
    raw_data = tf.convert_to_tensor(raw_data, name="raw_data", dtype=tf.int32)

    data_len = tf.size(raw_data)
    batch_len = data_len // batch_size
    data = tf.reshape(raw_data[0 : batch_size * batch_len],
                      [batch_size, batch_len])

    epoch_size = (batch_len - 1) // num_steps
    assertion = tf.assert_positive(
        epoch_size,
        message="epoch_size == 0, decrease batch_size or num_steps")
    with tf.control_dependencies([assertion]):
      epoch_size = tf.identity(epoch_size, name="epoch_size")

    i = tf.train.range_input_producer(epoch_size, shuffle=False).dequeue()
    x = tf.strided_slice(data, [0, i * num_steps],
                         [batch_size, (i + 1) * num_steps])
    x.set_shape([batch_size, num_steps])
    y = tf.strided_slice(data, [0, i * num_steps + 1],
                         [batch_size, (i + 1) * num_steps + 1])
    y.set_shape([batch_size, num_steps])
    return x, y


lyr_raw_data()
